import java.util.*;

public class PointTracker {

  // change this to adjust the number of grid points in
  // each direction
  static final int GRID_SIZE = 1000;

  class Point {
    public int x;
    public int y;

    /**
     * Constructs a point from grid coordinates.
     * 
     * @param x - horizontal coordinate.
     * @param y - vertical coordinate.
     */
    public Point(int x, int y) {
      this.x = x;
      this.y = y;
    }
  }

  // current point location
  private Point location;
  // TODO use some kind of data structure to track visited points

  /**
   * Constructs the position tracker and moves to
   * the middle of the grid.
   */
  PointTracker() {
    location = new Point(GRID_SIZE / 2, GRID_SIZE / 2);
    move(location);
  }

  /**
   * Determines if a point has already been visited.
   * 
   * @param point - Point object to check.
   * @return true if we already visited this point location.
   */
  public boolean haveBeenHere(Point point) {
    // TODO determine if we have been here already
    return true;
  }

  /**
   * Checks if a proposed point is out of bounds.
   * 
   * @param point - Point object to check.
   * @return - true if out-of-bounds, false if in-bounds.
   */
  public boolean outOfBounds(Point point) {
    if (point.x < 0 || point.x >= GRID_SIZE) {
      return true;
    }
    if (point.y < 0 || point.y >= GRID_SIZE) {
      return true;
    }
    return false;
  }

  /**
   * Save the current point location.
   * 
   * @param newLocation - updated location.
   */
  public void move(Point newLocation) {
    location = newLocation;
    // TODO keep track of this new point position.
  }

  /**
   * Returns a point in either +x, +y, -x, -y
   * directions (step of 1) from the current position.
   * 
   * @param d - direction to step (0: +x, 1: +y, 2: -x, 3: -y).
   * @return stepped point position.
   */
  public Point step(int d) {
    if (d == 0) {
      return new Point(location.x + 1, location.y);
    } else if (d == 1) {
      return new Point(location.x, location.y + 1);
    } else if (d == 2) {
      return new Point(location.x - 1, location.y);
    }
    return new Point(location.x, location.y - 1);
  }

  /**
   * Runs the stepping simulation.
   * 
   * @param args - not used.
   */
  public static void main(String[] args) {
    PointTracker tracker = new PointTracker();

    // the four possible directions we can take (+x, +y, -x, -y)
    // which will be shuffled during each step of the simulation
    ArrayList<Integer> directions = new ArrayList<>();
    directions.add(0);
    directions.add(1);
    directions.add(2);
    directions.add(3);

    int nSteps = 0;
    while (true) {

      // shuffle {0, 1, 2, 3} to take random direction attempts
      Collections.shuffle(directions);

      // try to step into each direction
      boolean moved = false;
      for (int k = 0; k < 4; k++) {
        int d = directions.get(k);
        Point newLocation = tracker.step(d); // get the coordinates for this attempt
        if (tracker.outOfBounds(newLocation)) // direction is not possible
          continue;
        if (tracker.haveBeenHere(newLocation)) // point has already been visited
          continue;
        tracker.move(newLocation); // move to and record the new location
        moved = true; // found a direction to move in
        nSteps++;
        break;
      }
      if (!moved)
        break; // no direction was found (we're stuck)
    }
    System.out.println("# steps = " + nSteps);
  }
}
